@@ -1,5 +1,5 @@
 /* This file is part of the 'stringi' package for R.
- * Copyright (c) 2013-2014, Marek Gagolewski and Bartek Tartanus
+ * Copyright (C) 2013-2016, Marek Gagolewski and Bartek Tartanus
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -57,6 +57,12 @@
  *
  * @version 0.3-1 (Marek Gagolewski, 2014-11-05)
  *    Issue #112: str_prepare_arg* retvals were not PROTECTed from gc
+ *
+ * @version 1.0-2 (Marek Gagolewski, 2016-01-29)
+ *    Issue #214: allow a regex pattern like `.*`  to match an empty string
+ *
+ * @version 1.0-2 (Marek Gagolewski, 2016-01-30)
+ *    Issue #210: Allow NA replacement
  */
 SEXP stri__replace_allfirstlast_regex(SEXP str, SEXP pattern, SEXP replacement, SEXP opts_regex, int type)
 {
@@ -78,18 +84,19 @@ SEXP stri__replace_allfirstlast_regex(SEXP str, SEXP pattern, SEXP replacement,
          i != pattern_cont.vectorize_end();
          i = pattern_cont.vectorize_next(i))
    {
-      STRI__CONTINUE_ON_EMPTY_OR_NA_STR_PATTERN(str_cont, pattern_cont,
-         SET_STRING_ELT(ret, i, NA_STRING);,
-         SET_STRING_ELT(ret, i, Rf_mkCharLenCE((const char*)NULL, 0, CE_UTF8));)
-
-      if (replacement_cont.isNA(i)) {
-         SET_STRING_ELT(ret, i, NA_STRING);
-         continue;
-      }
+      STRI__CONTINUE_ON_EMPTY_OR_NA_PATTERN(str_cont, pattern_cont,
+         SET_STRING_ELT(ret, i, NA_STRING);)
 
       RegexMatcher *matcher = pattern_cont.getMatcher(i); // will be deleted automatically
       matcher->reset(str_cont.get(i));
 
+      if (replacement_cont.isNA(i)) {
+         if (matcher->find())
+            str_cont.setNA(i);
+         SET_STRING_ELT(ret, i, str_cont.toR(i));
+         continue;
+      }
+
       UErrorCode status = U_ZERO_ERROR;
       if (type == 0) { // all
          str_cont.set(i, matcher->replaceAll(replacement_cont.get(i), status));
@@ -147,6 +154,9 @@ SEXP stri__replace_allfirstlast_regex(SEXP str, SEXP pattern, SEXP replacement,
  *
  * @version 0.3-1 (Marek Gagolewski, 2014-11-05)
  *    Issue #112: str_prepare_arg* retvals were not PROTECTed from gc
+ *
+ * @version 1.0-2 (Marek Gagolewski, 2016-01-30)
+ *    Issue #210: Allow NA replacement
  */
 SEXP stri__replace_all_regex_no_vectorize_all(SEXP str, SEXP pattern, SEXP replacement, SEXP opts_regex)
 { // version beta
@@ -186,7 +196,7 @@ SEXP stri__replace_all_regex_no_vectorize_all(SEXP str, SEXP pattern, SEXP repla
 
    for (R_len_t i = 0; i<pattern_n; ++i)
    {
-      if (pattern_cont.isNA(i) || replacement_cont.isNA(i)) {
+      if (pattern_cont.isNA(i)) {
          STRI__UNPROTECT_ALL
          return stri__vector_NA_strings(str_n);
       }
@@ -203,6 +213,12 @@ SEXP stri__replace_all_regex_no_vectorize_all(SEXP str, SEXP pattern, SEXP repla
 
          matcher->reset(str_cont.get(j));
 
+         if (replacement_cont.isNA(i)) {
+            if (matcher->find())
+               str_cont.setNA(j);
+            continue;
+         }
+
          UErrorCode status = U_ZERO_ERROR;
          str_cont.set(j, matcher->replaceAll(replacement_cont.get(i), status));
          STRI__CHECKICUSTATUS_THROW(status, {/* do nothing special on err */})
